@@ -17,6 +17,12 @@ import datetime as dt
 
 import matplotlib.pyplot as plt
 
+
+import read_vme_offline.general as general
+
+from shutil import copyfile
+
+
 #>>> import pandas as pd
 #>>> import matplotlib.pyplot as plt
 #>>> df=pd.read_hdf("run0001_20200526_102519.h5")
@@ -39,7 +45,7 @@ def shift_channels(df , TSHIFT, TWIN, x,y):
     return df
 
 
-def only_read( filename, x=0, y=1, batch=0, read_last=0):
+def only_read( filename, x=0, y=1, batch=0, read_last=0, MAXROWS=1000*1000):
     """
     reads the filename to DF and returns it
     """
@@ -70,26 +76,35 @@ def only_read( filename, x=0, y=1, batch=0, read_last=0):
 
     print("D... basename = ",basename)
     if read_last>0:
+        print("D... read_table last",read_last)
         df = pd.read_table( filename, names=['time',"e",'x','ch','y'],
-                        sep = "\s+", comment="#",
-                        nrows = read_last,
-                        skiprows=count-read_last)
+                            sep = "\s+", comment="#",
+                            nrows = read_last,
+                            skiprows=count-read_last,
+                            error_bad_lines=False,
+
+        )
                         #nrows = MAXROWS,
                         #skiprows=MAXROWS*batch)
     else:
+        print("D... read_table batch#",batch)
         df = pd.read_table( filename, names=['time',"e",'x','ch','y'],
                         sep = "\s+", comment="#",
                         nrows = MAXROWS,
                         skiprows=MAXROWS*batch)
 
+        print(df)
     return df
 
 
-def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = False):
+#----------------- used during the expseriment-----------------
+# plot is true by default to run from cmdline
+# overwrite when running from code
+def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = True):
     """
-    use: ./bin_readvme fast run0023_20200220_144753.asc 0 1  --read_last 500
+    COINCIDENCES: use: ./bin_readvme fast run0023_20200220_144753.asc 0 1  --read_last 500
     """
-    TSHIFT = 20 # 10 seems ok, 20 is safe (200ns)
+    TSHIFT = 30 # 10 seems ok, 20 is safe (200ns)  40 broke some detetors
     TWIN = 2*TSHIFT
     CHAN0=x
     CHAN1=y
@@ -99,7 +114,7 @@ def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = False
 
 
     if df is None:
-        df = only_read(filename, x,y,batch, read_last)
+        df = only_read(filename, x,y,batch, read_last, MAXROWS)
 
     # energy is marked "e"
     df = df.rename(columns={"e":ENE_0})
@@ -110,7 +125,7 @@ def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = False
         CONTINUE = False
     else:
         CONTINUE = True
-    print("D... len=", len(df))
+    print("D... len=", len(df)," SHIFTING chan IN TIME BY ",TSHIFT*10,"ns")
     df = shift_channels(df, TSHIFT, TWIN, x, y)
     print("D... len=", len(df) )
 
@@ -144,12 +159,12 @@ def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = False
 
 
     df1[ENE_1] = df1.shift(-1)[ENE_0]
-    # print(df1)
+    print(df1)
 
     print("D... dropping all when NEXT < ",TWIN )
     df2 = df1.loc[  df1["next"]<TWIN ]
     print( "D... len =",len(df2) )
-    print("D...  window mean / 3sigma ... {:.1f} +- {:.1f}".format(df2.next.mean(),  3*df2.next.std() ))
+    print("D...  window mean / 3sigma ... {:.1f} +- {:.1f}".format(df2["next"].mean(),  3*df2["next"].std() ))
 
 
     if CONTINUE:
@@ -158,7 +173,7 @@ def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = False
 
     if plot:
         df2.plot.scatter(x=ENE_0,  y=ENE_1,
-                     ylim=(0, 2000), xlim=(0,2000),
+                     ylim=(0, 6000), xlim=(0,6000),
                      s=.01);
         plt.show()
         return
@@ -168,79 +183,118 @@ def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = False
 
 
 
+# def get_number_of_lines(filename):
+#     ps = sp.Popen( ['cat', filename], stdout=sp.PIPE)
+#     output = sp.check_output(('wc', '-l'), stdin=ps.stdout)
+#     ps.wait()
+#     nlines=int(output.decode("utf8").split()[0])
+#     return nlines
+
+# def global_filename_decomp(filename):
+#     """
+#     The ONLY procedure to get the start time from filename
+#     """
+
+#     basename = os.path.basename(filename)
+#     basename = os.path.splitext(basename)[0]
+#     # start-date
+#     startd = basename.split("_")[1]
+#     # start-time
+#     startt = basename.split("_")[2]
+
+#     # 4 digits always
+#     if len(startd)==6:
+#         startd="20"+startd
+
+#     print("D...  time start MARK=",startd+startt)
+
+
+#     ok = False
+#     #try:
+#     start = dt.datetime.strptime(startd+startt,"%Y%m%d%H%M%S" )
+#     ok = True
+#     print("D... succesfull start with 4 digit year")
+#     #except:
+#     #    print("x... year may not by 4 digits")
+
+#     if not(ok):
+#         print("X... trying 2 digits for year")
+#         start = dt.datetime.strptime(startd+startt,"%y%m%d%H%M%S" )
+
+#     return start
 
 
 
 
-
-
-def main1(filename,  od=0, do=9999999 , chan=1):
+#=============================== MAIN1 EVA ==================
+def eva_cut_time(filename,  od=0, do=9999999 , chan=1):
     """
-    use: read_vme_offline cut1 filename_with_asc  60 120
+    EVA: use: read_vme_offline cut1 filename_with_asc  60 120
     """
     # od = 0
     # do = 999999
 
 
-    basename = os.path.basename(filename)
-    basename = os.path.splitext(basename)[0]
-    startd = basename.split("_")[1]
-    startt = basename.split("_")[2]
-    print("D...  time start MARK=",startd+startt)
+    # start = global_filename_decomp(filename)   # GLOBAL CALL
+    start = general.filename_decomp(filename)   # GLOBAL CALL
 
 
-    ok = False
-    #try:
-    start = dt.datetime.strptime(startd+startt,"%Y%m%d%H%M%S" )
-    ok = True
-    print("D... succesfull start with 4 digit year")
-    #except:
-    #    print("x... year may not by 4 digits")
-
-    if not(ok):
-        print("X... trying 2 digits for year")
-        start = dt.datetime.strptime(startd+startt,"%y%m%d%H%M%S" )
-
-
-
-    print("D... real start",start)
+    print(f"D... real start",start)
     od_dt = dt.timedelta(seconds=od)
     do_dt = dt.timedelta(seconds=do)
-    print(f"D... skip     {od} sec ",od_dt)
+    print(f"D... skip       {od} sec ... {od_dt}")
     startcut = start + od_dt
     stopcut = start + do_dt
-    print("D...  cut start",startcut)
-    print("D...  cut stop ",stopcut)
+    print(f"D... CUT  start {startcut}")
+    print(f"D... CUT  stop  {stopcut}  (demanded or max)")
 
-    print(basename)
-    df = pd.read_table( filename, names=['time','E','x','ch','y'], sep = "\s+", comment="#")
+
+    nlines = general.get_number_of_lines(filename)
+    print(f"i... reading the table of {nlines} lines ...")
+
+    # df = pd.read_table( filename, names=['time','E','x','ch','y'], sep = "\s+", comment="#")
+    df = general.pd_read_table(filename)
+    print(df.dtypes)
     print(df)
 
+    print()
     print(f"D...  selecting events for  channel {chan} from {od}s to {do}s")
+    print(f"D...  selecting events for  channel {chan} from {od}s to {do}s")
+    print(f"D...  selecting events for  channel {chan} from {od}s to {do}s")
+    print()
 
+    #------------------------------ 1 is a cut -----------------
     ex = 1e+8
     # df1 = df[ (df.ch==chan)&(df.E!=0)&(df.time>ex*od)&(df.time<ex*do)  ]
     df1 = df[ (df.ch==chan)&(df.time>ex*od)&(df.time<ex*do)  ]
     #df2=df['E']
     #df3 = df2[ (df1.E!=0)  ]
 
-
-    print(f"D...  numpy array for ch {chan}  ________________________CUT___________")
+    # print(f"D...  numpy array for ch {chan}  ________________________CUT___________")
     df1.reset_index(inplace=True, drop=True)
     print(df1)
 
-    dfzero = df1[ (df.E==0) ]
-    df2 = df1[ df.E!=0 ]
+
+    # dfzero = df1[ (df1.E==0) ]
+    len_dfzero = general.pd_detect_zeroes(df1) # df1[ (df1.E==0) ]
+
+    print()
+    print(f"D...  selecting nonzero events for  channel {chan} ")
+    print(f"D...  selecting nonzero events for  channel {chan} ")
+    print(f"D...  selecting nonzero events for  channel {chan} ")
+    print()
+
+    df2 = df1[ df1.E!=0 ]
     df2.reset_index(inplace=True, drop=True)
 
 
     print()
-    print("i... ZEROES == ", len(dfzero))
-    print("i... EVENTS == ", len(df2))
-    deadtpr = len(dfzero)/len(df2) * 100
+#    print("i... ZEROES == ", len(dfzero))
+#    print("i... EVENTS == ", len(df2))
+    deadtpr = len_dfzero/len(df2) * 100
     fev = df1.time.iloc[0]/ex
     lev = df1.time.iloc[-1]/ex
-    print(f"i... DT %   == {deadtpr:.2f}")
+#    print(f"i... DT %   == {deadtpr:.2f}")
     # print(f"i... events == {fev} ... {lev}")
     # print(f"i... times  == {fev:.2f} ... {lev:.2f}")
     dift = lev - fev
@@ -249,30 +303,56 @@ def main1(filename,  od=0, do=9999999 , chan=1):
 
     stopcut = start + dt.timedelta(seconds=lev)
 
-    print(f"i... times  == {fev:.2f} ... {lev:.2f}")
-    print(f"i... real T == {dift:.2f} s")
-    print(f"i... live T == {livet:.2f} s")
-    print(f"i... dead T == {deadt:.2f} s")
-    print(f"i... start  == {start} ")
-    print(f"i... CUTsta == {startcut} ")
-    print(f"i... CUTsto == {stopcut} ")
-    # print(f"i... stop   == {stop} ")
-    print()
+    output = f"""
+     file   == {filename}
+     times  == {fev:.2f} ... {lev:.2f}
+     real T == {dift:.2f} s
+     live T == {livet:.2f} s
+     dead T == {deadt:.2f} s
+     start  == {start}
+     CUTsta == {startcut}
+     CUTsto == {stopcut}
 
-    narr = df2.E.to_numpy()
+     zeroes     = {len_dfzero:8d}
+     nz events  = {len(df2):8d}
+     tot events = {len(df1):8d}
 
-    print(f"D...  creating histogram for channel {chan} from numpy")
+     DT %       = {deadtpr:.2f}
+"""
+    print(output)
 
-    his,bins = np.histogram(narr,bins=1024*32,range=(0.0,1024*32) )
-    print(his)
-    print(bins)
+    outinfo = os.path.splitext(filename)[0]+".info"
+    print(f"D... creating info FILE {outinfo}")
+    with open(outinfo,"w") as f:
+        f.write(output)
 
-    outfile = os.path.splitext(filename)[0]+".txt"
-    print("D... outfile=", outfile)
-    np.savetxt(outfile, his, fmt="%d")
+
+
+    bname = os.path.splitext(filename)[0]
+    outfile = bname+".txt"
+    hname = bname.split("_")[-1]
+    his = general.column_to_histo(df2['E'], savename = outfile, hname = hname)
+    outfile2 = bname+".asc1"
+    # his = general.column_to_histo(df2["E"], savename = outfile, hname = hname)
+    copyfile(outfile, outfile2)
+
+    #narr = df2.E.to_numpy()
+    #    print("D... to_numpy", len(narr), narr.dtype)
+    #print(narr)
+
+    #print(f"D...  creating histogram for channel {chan} from numpy")
+
+    #his,bins = np.histogram(narr,bins=1024*32,range=(0.0,1024*32) )
+    #print()
+    #print(his)
+    #print(bins)
+    #print()
+
+    #print(f"D... creating text spectrum = {outfile}")
+    #np.savetxt(outfile, his, fmt="%d")
 
 
 
 if __name__=="__main__":
     print("D... fastread can be called too, from bin_readvme")
-    Fire(main1)
+    Fire(eva_cut_time)
